/* ens_dch.c
 * Fichier source de l'implantation des ensembles d'entiers
 * sous la forme d'un tableau de booleens
 * Auteur : VOURIOT Thierry
 * Date : 24 novembre 2001
 */

#include "ens_dch.h"

/* Creation d'un ensemble vide */
Ens ensnouv(void)
{
	Ens res = NULL;
	return res;
}

/* Insertion d'un entier 
 * pre ins(e,x) = 1<=x<=N et !app(e,x)
 */
Ens ins(Ens e, Nat k)
{
	Ens res = MALLOC(SEns);
	res->v = k;
	
	if (e != NULL) {
		res->s = e;
		res->p = e->p;
		e->p->s = res;
		e->p = res;
	}
	else {
		res->s = res;
		res->p = res;
	}
	return res;
}
	
/* Creation d'un ensemble avec un entier 
 * pre sing(x) = 1<=x<=N
 */
Ens sing(Nat k)
{
	return ins(ensnouv(),k);
}

/* Suppression d'un entier 
 * pre supp(e,x) = 1<=x<=N et app(e,x)
 */
Ens supp(Ens e, Nat k)
{
	Ens res;
	Bool suite = TRUE;
	
	/* Si l'ensemble ne contient q'un element on retourne l'ensemble vide */
	if (e->s == e)
	{
		return NULL;
		free(e);
	}
	/* On positionne la tete de la liste sur l'element a supprimer */
	while (suite)
	{
		if (e->v == k)
			suite = FALSE;
		else e = e->s;
	}
	
	res = e->s;
	res->p = e->p;
	e->p->s = res;

	free(e);

	return res;
}

/* Teste si un entier appartient a l'ensemble 
 * pre app(e,x) = 1<=x<=N
 */
Bool app(Ens e, Nat k)
{
	Ens res = e;
	Bool trouve = FALSE;

	if (res != NULL)
	{
		if (res->v == k)
			trouve = TRUE;
		
		for (res=res->s;res!=e && !trouve;res=res->s)
		{	
			if (res->v == k)
				trouve = TRUE;
		}
	}
	return trouve;
}

/* Retourne l'union de 2 ensemble */
Ens unio(Ens e1, Ens e2)
{
	Ens res = ensnouv();
	
	/* Copie de e1 dans res */
	Ens tmp = e1;	
	if (e1 != NULL)
	{
		res = ins(res,tmp->v);
		for (tmp=tmp->s;tmp!=e1;tmp=tmp->s)
		{	
			res = ins(res,tmp->v);
		}
	}
	
	/* Copie dans res des elements de e2 n'appartenant pas a e1 */
	tmp = e2;	
	if (e2 != NULL)
	{
		if (!app(res,tmp->v))
			res = ins(res,tmp->v);
		for (tmp=tmp->s;tmp!=e2;tmp=tmp->s)
		{	
			if (!app(res,tmp->v))
				res = ins(res,tmp->v);
		}
	}
	
	return res;
}

/* Retourne l'intersection de 2 ensemble */
Ens inter(Ens e1, Ens e2)
{
	Ens res = ensnouv();
	
	Ens tmp = e1;	
	if (e1 != NULL)
	{
		if (app(e2,tmp->v))
			res = ins(res,tmp->v);
		for (tmp=tmp->s;tmp!=e1;tmp=tmp->s)
		{	
			if (app(e2,tmp->v))
				res = ins(res,tmp->v);
		}
	}
	return res;
}

/* Creation de l'ensemble "plein" */
Ens ref(void)
{
	Ens res = ensnouv();
	int i;
	
	for (i=1;i<=N;i++)
		res = ins(res,i);
	
	return res;
}

/* Cardinalite (nb d'elements) de l'ensemble */
Nat card(Ens e)
{
	int res=0;	
	Ens tmp = e;	
	if (e != NULL)
	{
		res=1;
		for (tmp=tmp->s;tmp!=e;tmp=tmp->s)
		{	
			res++;
		}
	}
	return res;
}

/* Teste si un ensemble est vide */
Bool vide(Ens e) {
	
	return e == NULL;
}

/* Teste si le second enemble est inclus dans le premier */
Bool inclu(Ens e1, Ens e2) {
	Bool res = TRUE;
	Ens tmp = e2;	
	if (e2 != NULL)
	{
		if (!app(e1,tmp->v))
			res = FALSE;
		for (tmp=tmp->s;tmp!=e2 && res;tmp=tmp->s)
		{	
			if (!app(e1,tmp->v))
				res = FALSE;	
		}
	}
	return res;
}

/* Teste l'egalite de 2 ensembles */
Bool eg(Ens e1, Ens e2) {
	
	return inclu(e1,e2) && inclu(e2,e1);
}

/* Retourne le complementaire d'un ensemble */
Ens comp(Ens e)
{	
	return diff(ref(),e);
}
	
/* Retourne la difference de 2 ensembles */
Ens diff(Ens e1, Ens e2)
{
	Ens res = ensnouv();
	/* Copie de e1 dans res */
	Ens tmp = e1;	
	if (e1 != NULL)
	{
		res = ins(res,tmp->v);
		for (tmp=tmp->s;tmp!=e1;tmp=tmp->s)
		{	
			res = ins(res,tmp->v);
		}
	}
	tmp = e2;	
	if (e2 != NULL)
	{
		if (app(res,tmp->v))
			res = supp(res,tmp->v);
		for (tmp=tmp->s;tmp!=e2;tmp=tmp->s)
		{	
			if (app(res,tmp->v))
				res = supp(res,tmp->v);	
		}
	}
	return res;
}

/* Affiche sur la sortie standard les valeurs de l'ensemble */
void aff(Ens e)
{
	Ens res = e;
	printf("[ ");
	
	if (res != NULL)
	{		
		for (res=res->p;res!=e;res=res->p)
		{	
			printf("%d ",res->v);
		}
		printf("%d ",res->v);
	}
	printf("]\n");
}

/* Lit un ensemble au clavier */
Ens lirens(void)
{
	Ens res = ensnouv();
	int k;

	scanf("%d",&k);getchar();
	while (k>0)
	{
		if (k<=N && k>=1 && !app(res,k))
			res = ins(res,k);
		scanf("%d",&k);getchar();
		
	}
	return res;
}

