/* ens_dch.h
 * Fichier d'en tete pour l'implantation d'un ensemble d'entier de 1  N
 * Auteur : VOURIOT Thierry
 * Date : 24 novembre 2001
 */

#include "base.h"

/* Taille de l'ensemble */
#define N 50

/* Implantation sous la forme d'une liste doublement chainees */
typedef struct sEns
{
	struct sEns *p;		/* Predecesseur */
	Nat v;			/* Valeur */
	struct sEns *s;		/* Successeur */
} SEns, *Ens;


/* Creation d'un ensemble vide */
extern Ens ensnouv(void);

/* Insertion d'un entier 
 * pre ins(e,x) = 1<=x<=N et !app(e,x)
 */
extern Ens ins(Ens, Nat);

/* Creation d'un ensemble avec un entier 
 * pre sing(x) = 1<=x<=N
 */
extern Ens sing(Nat);

/* Suppression d'un entier 
 * pre supp(e,x) = 1<=x<=N et app(e,x)
 */
extern Ens supp(Ens, Nat);

/* Teste si un entier appartient a l'ensemble 
 * pre app(e,x) = 1<=x<=N
 */
extern Bool app(Ens, Nat);

/* Retourne l'union de 2 ensemble */
extern Ens unio(Ens, Ens);

/* Retourne l'intersection de 2 ensemble */
extern Ens inter(Ens, Ens);

/* Creation de l'ensemble "plein" */
extern Ens ref(void);

/* Cardinalite (nb d'elements) de l'ensemble */
extern Nat card(Ens);

/* Teste si un ensemble est vide */
extern Bool vide(Ens);

/* Teste si le second enemble est inclus dans le premier */
extern Bool inclu(Ens, Ens);

/* Teste l'egalite de 2 ensembles */
extern Bool eg(Ens, Ens);

/* Retourne le complementaire d'un ensemble */
extern Ens comp(Ens);

/* Retourne la difference de 2 ensembles */
extern Ens diff(Ens, Ens);

/* Affiche sur la sortie standard les valeurs de l'ensemble */
extern void aff(Ens e);

/* Lit un ensemble au clavier */
extern Ens lirens(void);





