/* ens_tab.c
 * Fichier source de l'implantation des ensembles d'entiers
 * sous la forme d'un tableau de booleens
 * Auteur : VOURIOT Thierry
 * Date : 14 novembre 2001
 */

#include "ens_tab.h"

/* Creation d'un ensemble vide */
Ens ensnouv(void)
{
	int i;
	Ens e;
	for (i=0;i<N;i++)
		e.v[i] = false;

	return e;
}

/* Insertion d'un entier 
 * pre ins(e,x) = 1<=x<=N et !app(e,x)
 */
Ens ins(Ens e, Nat k)
{
	Ens res = ensnouv();
	int i;

	for (i=0;i<N;i++)
		res.v[i] = e.v[i];

	res.v[k-1] = true;
	return res;
}
	
/* Creation d'un ensemble avec un entier 
 * pre sing(x) = 1<=x<=N
 */
Ens sing(Nat k)
{
	Ens res = ensnouv();
	res.v[k-1] = true;
	return res;
}

/* Suppression d'un entier 
 * pre supp(e,x) = 1<=x<=N et app(e,x)
 */
Ens supp(Ens e, Nat k)
{
	Ens res = ensnouv();
	int i;

	for (i=0;i<N;i++)
		res.v[i] = e.v[i];

	res.v[k-1] = false;
	return res;
}

/* Teste si un entier appartient a l'ensemble 
 * pre app(e,x) = 1<=x<=N
 */
Bool app(Ens e, Nat k)
{
	return e.v[k-1];
}

/* Retourne l'union de 2 ensemble */
Ens unio(Ens e1, Ens e2)
{
	Ens res = ensnouv();
	int i;
	for (i=0;i<N;i++)
		res.v[i] = e1.v[i] || e2.v[i];
	return res;
}

/* Retourne l'intersection de 2 ensemble */
Ens inter(Ens e1, Ens e2)
{
	Ens res = ensnouv();
	int i;
	for (i=0;i<N;i++)
		res.v[i] = e1.v[i] && e2.v[i];
	return res;
}

/* Creation de l'ensemble "plein" */
Ens ref(void)
{
	int i;
	Ens e;
	for (i=0;i<N;i++)
		e.v[i] = true;

	return e;
}

/* Cardinalite (nb d'elements) de l'ensemble */
Nat card(Ens e)
{
	int res=0;	
	int i;
	for (i=0;i<N;i++)
		if (e.v[i] == true)
			res++;
	return res;
}

/* Teste si un ensemble est vide */
Bool vide(Ens e) {
	Bool res = true;
	int i;
	for (i=0;i<N;i++) 
		if (e.v[i]){res = false;break;}
	return res;
}

/* Teste si le second enemble est inclus dans le premier */
Bool inclu(Ens e1, Ens e2) {
	Bool res = true;
	int i;

	for (i=0;i<N;i++) 
		if (e2.v[i] && !e1.v[i])
			{ res = false; break; }		
	return res;
}

/* Teste l'egalite de 2 ensembles */
Bool eg(Ens e1, Ens e2) {
	Bool res = true;
	int i;
	for (i=0;i<N;i++) 
		if (e2.v[i] != e1.v[i])
			{ res = false; break; }	
	return res;
}

/* Retourne le complementaire d'un ensemble */
Ens comp(Ens e)
{
	Ens res = ensnouv();
	int i;
	for (i=0;i<N;i++)
		res.v[i] = !e.v[i];
	return res;
}
	
/* Retourne la difference de 2 ensembles */
Ens diff(Ens e1, Ens e2)
{
	Ens res = ensnouv();
	int i;
	for (i=0;i<N;i++)
		if (e1.v[i])
			if (e2.v[i])
				res.v[i] = false;
			else res.v[i] = true;
		else res.v[i] = false;

	return res;
}
/* Copie d'un ensemble */
Ens copie(Ens e)
{
	return unio(e,ensnouv());
}

/* Affiche sur la sortie standard les valeurs de l'ensemble */
void aff(Ens e)
{
	int i;
	printf("[ ");
	for (i=0;i<N;i++)	
		if (e.v[i]) printf("%d ",i+1);	
	printf("]\n");
}

/* Lit un ensemble au clavier */
Ens lirens(void)
{
	Ens res = ensnouv();
	int k;

	scanf("%d",&k);getchar();
	while (k>0)
	{
		if (k<=N && k>=1 && !app(res,k))
			res = ins(res,k);
		scanf("%d",&k);getchar();
		
	}
	return res;
}

